"use strict";
// Copyright 2021 Google Inc. Use of this source code is governed by an
// MIT-style license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.
Object.defineProperty(exports, "__esModule", { value: true });
exports.render = void 0;
const p = require("path");
const url_1 = require("url");
const exception_1 = require("./exception");
const compile_1 = require("./compile");
const utils_1 = require("./utils");
function render(options, callback) {
    if (!('file' in options && options.file) && !('data' in options)) {
        callback(newLegacyException(new Error('Either options.data or options.file must be set.')));
        return;
    }
    const start = Date.now();
    const compileSass = isStringOptions(options)
        ? (0, compile_1.compileStringAsync)(options.data, {
            sourceMap: wasSourceMapRequested(options),
            url: options.file ? (0, url_1.pathToFileURL)(options.file) : undefined,
            syntax: options.indentedSyntax ? 'indented' : 'scss',
        })
        : (0, compile_1.compileAsync)(options.file, {
            sourceMap: wasSourceMapRequested(options),
            loadPaths: options.includePaths,
        });
    compileSass.then(result => callback(undefined, newLegacyResult(options, start, result)), error => callback(newLegacyException(error)));
}
exports.render = render;
// Returns whether `options` is a `LegacyStringOptions`.
function isStringOptions(options) {
    return 'data' in options;
}
// Determines whether a sourceMap was requested by the call to `render()`.
function wasSourceMapRequested(options) {
    return (typeof options.sourceMap === 'string' ||
        (options.sourceMap === true && !!options.outFile));
}
// Transforms the compilation result into an object that mimics the Node Sass
// API format.
function newLegacyResult(options, start, result) {
    var _a, _b;
    const end = Date.now();
    let css = result.css;
    let sourceMapBytes;
    if (result.sourceMap) {
        const sourceMap = result.sourceMap;
        sourceMap.sourceRoot = (_a = options.sourceMapRoot) !== null && _a !== void 0 ? _a : '';
        const sourceMapPath = typeof options.sourceMap === 'string'
            ? options.sourceMap
            : options.outFile + '.map';
        const sourceMapDir = p.dirname(sourceMapPath);
        if (options.outFile) {
            sourceMap.file = (0, utils_1.pathToUrlString)(p.relative(sourceMapDir, options.outFile));
        }
        else if (options.file) {
            sourceMap.file = (0, utils_1.pathToUrlString)(p.relative(sourceMapDir, (0, utils_1.withoutExtension)(options.file) + '.css'));
        }
        else {
            sourceMap.file = 'stdin.css';
        }
        sourceMap.sources = sourceMap.sources.map(source => {
            if (source.startsWith('file://')) {
                return (0, utils_1.pathToUrlString)(p.relative(sourceMapDir, (0, url_1.fileURLToPath)(source)));
            }
            else if (source.startsWith('data:')) {
                return 'stdin';
            }
            else {
                return source;
            }
        });
        sourceMapBytes = Buffer.from(JSON.stringify(sourceMap));
        if (!options.omitSourceMapUrl) {
            let url;
            if (options.sourceMapEmbed) {
                url = `data:application/json;base64,${sourceMapBytes.toString('base64')}`;
            }
            else if (options.outFile) {
                url = (0, utils_1.pathToUrlString)(p.relative(p.dirname(options.outFile), sourceMapPath));
            }
            else {
                url = (0, utils_1.pathToUrlString)(sourceMapPath);
            }
            css += `\n\n/*# sourceMappingURL=${url} */`;
        }
    }
    return {
        css: Buffer.from(css),
        map: sourceMapBytes,
        stats: {
            entry: (_b = options.file) !== null && _b !== void 0 ? _b : 'data',
            start,
            end,
            duration: end - start,
            includedFiles: result.loadedUrls.map(url => url.toString()),
        },
    };
}
// Decorates an Error with additional fields so that it behaves like a Node Sass
// error.
function newLegacyException(error) {
    var _a, _b, _c, _d;
    if (!(error instanceof exception_1.Exception)) {
        return Object.assign(error, {
            formatted: error.toString(),
            status: 3,
        });
    }
    const file = ((_a = error.span) === null || _a === void 0 ? void 0 : _a.url)
        ? // We have to cast to Node's URL type here because the specified type is the
            // standard URL type which is slightly less featureful. `fileURLToPath()`
            // does work with standard URL objects in practice, but we know that we
            // generate Node URLs here regardless.
            (0, url_1.fileURLToPath)((_b = error.span) === null || _b === void 0 ? void 0 : _b.url)
        : 'stdin';
    return Object.assign(new Error(), {
        status: 1,
        message: error.toString().replace(/^Error: /, ''),
        formatted: error.toString(),
        toString: () => error.toString(),
        stack: error.stack,
        line: (0, utils_1.isNullOrUndefined)((_c = error.span) === null || _c === void 0 ? void 0 : _c.start.line)
            ? undefined
            : error.span.start.line + 1,
        column: (0, utils_1.isNullOrUndefined)((_d = error.span) === null || _d === void 0 ? void 0 : _d.start.column)
            ? undefined
            : error.span.start.column + 1,
        file,
    });
}
//# sourceMappingURL=legacy.js.map